/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.smilecdr.demo.fhirendpoint;

import ca.cdr.api.model.json.ICdrEndpointRequestDetailsJson;
import ca.uhn.fhir.interceptor.api.Hook;
import ca.uhn.fhir.interceptor.api.Interceptor;
import ca.uhn.fhir.interceptor.api.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import java.util.Iterator;
import java.util.function.Supplier;

/**
 * This interceptor permits to modify the header information saved in Audit Log, available in web admin console.
 */
@SuppressWarnings({"unused"})
@Interceptor
public class AuditLogHeaderUpdateInterceptor {

	private static final Logger ourLog = LoggerFactory.getLogger(AuditLogHeaderUpdateInterceptor.class);

	// inject supplier of request details that are used to build Audit Log entry
	@Autowired
	Supplier<ICdrEndpointRequestDetailsJson> theCdrEndpointRequestDetailsJsonSupplier;

	@Hook(Pointcut.SERVER_INCOMING_REQUEST_PRE_PROCESSED)
	public boolean serverIncomingRequestPreProcessed(
		HttpServletRequest theHttpServletRequest,
		HttpServletResponse theHttpServletResponse) {

		ourLog.info("AuditLogHeaderUpdateInterceptor SERVER_INCOMING_REQUEST_PRE_PROCESSED - started");

		// for demonstration purposes
		// get the http request headers, output them in the system log
		Iterator<String> headerNamesIterator = theHttpServletRequest.getHeaderNames().asIterator();
		headerNamesIterator.forEachRemaining(headerName -> theHttpServletRequest.getHeaders(headerName).asIterator()
			.forEachRemaining(headerValue -> ourLog.info("http request header: {} value: {}", headerName, headerValue)));

		// add a new header to be saved in the Audit Log
		ICdrEndpointRequestDetailsJson details = theCdrEndpointRequestDetailsJsonSupplier.get();
		details.getHeaders().put("MyCustomHeader", "MyImportantHeaderValue");

		// output in the system log the headers that will be present in the Audit Log entry created for this request
		details.getHeaders().forEach((headerName, headerValue) ->
			ourLog.info("request details header: {} value: {}", headerName, headerValue));

		ourLog.info("AuditLogHeaderUpdateInterceptor SERVER_INCOMING_REQUEST_PRE_PROCESSED - ended");

		return true;
	}

}
