/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.smilecdr.demo.fhirgateway;

import ca.cdr.api.fhir.interceptor.CdrHook;
import ca.cdr.api.fhir.interceptor.CdrPointcut;
import ca.cdr.api.fhirgw.json.GatewayTargetJson;
import ca.cdr.api.fhirgw.model.ISearchResultsAccumulator;
import ca.cdr.api.fhirgw.model.SearchResponse;
import ca.uhn.fhir.interceptor.api.Interceptor;
import org.hl7.fhir.instance.model.api.IBaseResource;
import org.hl7.fhir.instance.model.api.IIdType;
import org.hl7.fhir.r4.model.IdType;

/**
 * This interceptor is intended to be used in a FHIR Gateway that is configured with multiple
 * search targets. It modifies the results of searches from a specific target, changing the
 * way that they will be presented to Gateway clients.
 *
 * In this example, we are simply making a basic change to the ID of the
 * resources being returned. You could make other changes however, such
 * as adding or removing elements, or even removing result resources
 * entirely.
 *
 * If you need to see reults from previous searches in order to inform changes being
 * made here, you can retrieve these from {@literal theSearchResultsAccumulator}.
 */
@Interceptor
public class GatewayEnrichGatewayTargetResponses {

	@CdrHook(CdrPointcut.FHIRGW_SEARCH_TARGET_POSTINVOKE)
	public void hook(GatewayTargetJson theTarget, ISearchResultsAccumulator theISearchResultsAccumulator, SearchResponse theResponse) {

		// We assume there are two targets, one called "target1" and one called "target2". This
		// logic only applies to the second target.
		if (!theTarget.getId().equals("target1")) {
			return;
		}

		// Loop through the results of the search and modify them
		for (IBaseResource next : theResponse.getSearchResults()) {
			IIdType existingId = next.getIdElement();
			IdType newId = new IdType("1000" + existingId.getIdPart());
			next.setId(newId);
		}
	}

}
